@@ -19,7 +19,7 @@
 
 
 __author__ = "Sandro Gauci <sandro@enablesecurity.com>"
-__version__ = '0.3.3'
+__version__ = '0.3.4'
 
 
 import re
@@ -31,6 +31,7 @@ import socket
 import random
 import struct
 import shutil
+import optparse
 import logging
 from random import getrandbits
 from urllib.request import urlopen
@@ -46,6 +47,13 @@ if sys.hexversion < 0x03060000:
     sys.exit(1)
 
 
+class ArgumentParser(optparse.OptionParser):
+    def error(self, message, code=10):
+        print(self.get_usage())
+        sys.stderr.write('error: %s\r\n' % message)
+        sys.exit(code)
+
+
 def standardoptions(parser):
     parser.add_option('-v', '--verbose', dest="verbose", action="count",
                       help="Increase verbosity")
@@ -77,6 +85,12 @@ def standardscanneroptions(parser):
     return parser
 
 
+def resolveexitcode(newint, existingcode):
+    if existingcode > newint:
+        return existingcode
+    return newint
+
+
 def calcloglevel(options):
     logginglevel = 30
     if options.verbose is not None:
@@ -704,6 +718,7 @@ def ip6range(*args):
 
 
 def getranges(ipstring):
+    from sipvicious import svmap
     log = logging.getLogger('getranges')
     if re.match(
         r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}-\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
@@ -730,8 +745,15 @@ def getranges(ipstring):
             naddr2 = naddr1
         except socket.error:
             log.error('Could not resolve %s' % ipstring)
+            svmap.__exitcode__ = 30  # network error
             return
-    return((naddr1, naddr2))
+        # UnicodeError is raised by the idna library when a malformed IP
+        # is passed to socket.gethostbyname(). e.g. gethostbyname('1.1..1')
+        except UnicodeError:
+            log.error('Malformed target supplied: %s' % ipstring)
+            svmap.__exitcode__ = resolveexitcode(10, svmap.__exitcode__)
+            return
+    return naddr1, naddr2
 
 
 def getranges2(ipstring):
@@ -948,13 +970,13 @@ def outputtoxml(title, labels, db, resdb=None, xsl='resources/sv.xsl'):
     for k in db.keys():
         o += '<result>\r\n'
         o += '<%s><value>%s</value></%s>\r\n' % (labels[0].replace(
-            ' ', '').lower(), k, escape(labels[0]).replace(' ', '').lower())
+            ' ', '').lower(), k.decode(), escape(labels[0]).replace(' ', '').lower())
         o += '<%s><value>%s</value></%s>\r\n' % (labels[1].replace(
-            ' ', '').lower(), escape(db[k]), labels[1].replace(' ', '').lower())
+            ' ', '').lower(), escape(db[k].decode()), labels[1].replace(' ', '').lower())
         if resdb is not None:
             if k in resdb:
                 o += '<%s><value>%s</value></%s>\r\n' % (labels[2].replace(
-                    ' ', '').lower(), escape(resdb[k]), labels[2].replace(' ', '').lower())
+                    ' ', '').lower(), escape(resdb[k].decode()), labels[2].replace(' ', '').lower())
             else:
                 o += '<%s><value>N/A</value></%s>\r\n' % (labels[2].replace(
                     ' ', '').lower(), labels[2].replace(' ', '').lower())
@@ -1003,7 +1025,6 @@ def outputtopdf(outputfile, title, labels, db, resdb):
         from reportlab.platypus import TableStyle, Table, SimpleDocTemplate, Paragraph
         from reportlab.lib import colors
         from reportlab.lib.styles import getSampleStyleSheet
-        from reportlab.pdfgen import canvas
     except ImportError:
         log.error(
             'Reportlab was not found. To export to pdf you need to have reportlab installed. Check out www.reportlab.org')
